import React, { HTMLProps, useEffect, useMemo, useState } from "react";
import { Company, formatAddress, Office } from "@/data";
import { Autocomplete } from "@material-ui/lab";
import { Grid, TextField, Typography } from "@material-ui/core";
import { InternshipFormValues } from "@/forms/internship";
import { useTranslation } from "react-i18next";
import { Field, useFormikContext } from "formik";
import { TextField as TextFieldFormik } from "formik-material-ui"
import api from "@/api";

export const CompanyItem = ({ company, ...props }: { company: Company } & HTMLProps<any>) => (
    <div className="company-item" { ...props }>
        <div>{ company.name }</div>
        <Typography variant="caption">NIP: <strong>{ company.nip }</strong></Typography>
    </div>
)

export const OfficeItem = ({ office, ...props }: { office: Office } & HTMLProps<any>) => (
    <div className="office-item" { ...props }>
        <div>{ office.address.city }</div>
        <Typography variant="caption">{ formatAddress(office.address) }</Typography>
    </div>
)

export const BranchForm: React.FC = () => {
    const { values, errors, setValues, touched, setFieldTouched } = useFormikContext<InternshipFormValues>();
    const { t } = useTranslation();

    const disabled = useMemo(() => !values.companyName, [values.companyName]);
    const [offices, setOffices] = useState<Office[]>([]);
    const canEdit = useMemo(() => !values.office && !disabled, [values.office, disabled]);

    useEffect(() => {
        (async () => {
            setOffices(values.company?.id ? (await api.companies.offices(values.company?.id)) : []);
        })()
    }, [ values.company?.id ])

    const handleCityChange = (event: any, value: Office | string | null) => {
        if (typeof value === "string") {
            setValues({
                ...values,
                office: null,
                city: value,
            }, true);
        } else if (typeof value === "object" && value !== null) {
            const office = value as Office;

            setValues({
                ...values,
                office,
                city: office.address.city,
                country: office.address.country,
                street: office.address.street,
                building: office.address.building,
                postalCode: office.address.postalCode,
            }, true)
        } else { // null
            setValues({
                ...values,
                office: null,
                city: "",
                country: "",
                street: "",
                building: "",
                postalCode: "",
            }, true)
        }
    }

    const handleCityInput = (event: any, value: string) => {
        setValues( {
            ...values,
            office: null,
            ...(values.office ? {
                country: "",
                street: "",
                building: "",
                postalCode: "",
            } : { }),
            city: value,
        }, true);
    }

    return (
        <div>
            <Grid container>
                <Grid item md={ 7 }>
                    <Autocomplete options={ offices || [] }
                                  disabled={ disabled }
                                  getOptionLabel={ office => typeof office == "string" ? office : office.address.city }
                                  renderOption={ office => <OfficeItem office={ office }/> }
                                  renderInput={
                                      props =>
                                          <TextField { ...props }
                                                     label={ t("forms.internship.fields.city") }
                                                     fullWidth
                                                     error={ touched.city && !!errors.city }
                                                     helperText={ touched.city && errors.city }
                                          />
                                  }
                                  onChange={ handleCityChange }
                                  onInputChange={ handleCityInput }
                                  onBlur={ ev => setFieldTouched("city", true) }
                                  inputValue={ values.city }
                                  value={ values.office ? values.office : values.city }
                                  freeSolo
                    />
                </Grid>
                <Grid item md={ 2 }>
                    <Field label={ t("forms.internship.fields.postal-code") } name="postalCode" fullWidth disabled={ !canEdit } component={ TextFieldFormik }/>
                </Grid>
                <Grid item md={ 3 }>
                    <Field label={ t("forms.internship.fields.country") } name="country" fullWidth disabled={ !canEdit } component={ TextFieldFormik }/>
                </Grid>
                <Grid item md={ 10 }>
                    <Field label={ t("forms.internship.fields.street") } name="street" fullWidth disabled={ !canEdit } component={ TextFieldFormik }/>
                </Grid>
                <Grid item md={ 2 }>
                    <Field label={ t("forms.internship.fields.building") } name="building" fullWidth disabled={ !canEdit } component={ TextFieldFormik }/>
                </Grid>
            </Grid>
        </div>
    )
}

export const MentorForm = () => {
    const { t } = useTranslation();

    return (
        <Grid container>
            <Grid item md={ 6 }>
                <Field name="mentorFirstName" label={ t("forms.internship.fields.first-name") } fullWidth component={ TextFieldFormik }/>
            </Grid>
            <Grid item md={ 6 }>
                <Field name="mentorLastName" label={ t("forms.internship.fields.last-name") } fullWidth component={ TextFieldFormik }/>
            </Grid>
            <Grid item md={ 8 }>
                <Field name="mentorEmail" label={ t("forms.internship.fields.e-mail") } fullWidth component={ TextFieldFormik }/>
            </Grid>
            <Grid item md={ 4 }>
                <Field name="mentorPhone" label={ t("forms.internship.fields.phone") } fullWidth component={ TextFieldFormik }/>
            </Grid>
        </Grid>
    );
}

export const CompanyForm: React.FunctionComponent = () => {
    const { values, setValues, errors, touched, setFieldTouched } = useFormikContext<InternshipFormValues>();
    const { t } = useTranslation();

    const [input, setInput] = useState<string>("");
    const [companies, setCompanies] = useState<Company[]>([]);

    const canEdit = useMemo(() => !values.company, [values.company]);

    useEffect(() => {
        if (!input || values.companyName == input) {
            return;
        }

        (async () => {
            setCompanies(await api.companies.search(input));
        })()

        setValues({ ...values, company: null, companyName: input }, true)
    }, [ input ]);

    const handleCompanyChange = (event: any, value: Company | string | null) => {
        setFieldTouched("companyName", true);

        if (typeof value === "string") {
            setValues({
                ...values,
                company: null,
                companyName: value
            }, true)
        } else if (typeof value === "object" && value !== null) {
            setValues({
                ...values,
                company: value as Company,
                companyName: value.name,
                companyNip: value.nip,
            }, true)
        } else {
            setValues({
                ...values,
                company: null,
                companyName: "",
            }, true);
        }
    }

    return (
        <>
            <Grid container>
                <Grid item xs={12}>
                    <Autocomplete options={ companies }
                                  getOptionLabel={ option => typeof option === "string" ? option : option.name }
                                  renderOption={ company => <CompanyItem company={ company }/> }
                                  renderInput={ props => <TextField { ...props } label={ t("forms.internship.fields.company-name") } fullWidth
                                                                    error={ touched.companyName && !!errors.companyName } helperText={ touched.companyName && errors.companyName }/> }
                                  onChange={ handleCompanyChange }
                                  value={ values.company || values.companyName }
                                  inputValue={ input }
                                  freeSolo
                                  onInputChange={ (_, value) => setInput(value) }
                    />
                </Grid>
                <Grid item md={ 4 }>
                    <Field label={ t("forms.internship.fields.nip") } fullWidth name="companyNip" disabled={ !canEdit } component={ TextFieldFormik }/>
                </Grid>
            </Grid>
            <Typography variant="subtitle1" className="subsection-header">{ t("internship.mentor") }</Typography>
            <MentorForm/>
            <Typography variant="subtitle1" className="subsection-header">{ t("internship.office") }</Typography>
            <BranchForm/>
        </>
    )
}

